--[[
RegisterSpecialization

Author:		Ifko[nator]
Date:		18.11.2021
Version:	2.1 (by Sushifresser)

History:	v1.0 @02.01.2019 - initial implementation in FS 19
			---------------------------------------------------
			v2.0 @18.11.2021 - convert to FS 22
			v2.1 @19.12.2022 - bugfix in call to "addSpecialization" (line 80). removed excess attribute
]]

RegisterSpecialization = {};
RegisterSpecialization.currentModDirectory = g_currentModDirectory;

local modName = g_currentModName;
local modDesc = loadXMLFile("modDesc", RegisterSpecialization.currentModDirectory .. "modDesc.xml");
local modDir = g_currentModDirectory;

RegisterSpecialization.debugPriority = Utils.getNoNil(getXMLInt(modDesc, "modDesc.registerSpecializations#debugPriority"), 0);


function printDebug(debugMessage, priority, addString)
	if RegisterSpecialization.debugPriority >= priority then
		local prefix = "";
		
		if addString then
			prefix = "::DEBUG:: from the RegisterSpecialization.lua: ";
		end;
		
		print(prefix .. tostring(debugMessage));
	end;
end;
--[[
if isDischargeable and hasFillUnit and hasWheels and hasFillVolume and not isBaler and not isConveyorBelt then					
--]]

local specializationRegister = function(specializationName, specializationClassName, specializationFilename, searchedSpecializations, blockSpecializations)
	local neededSpec = 0;
    local hasForNeed = 0;
	local specializationObject = g_specializationManager:getSpecializationObjectByName(modName .. "."..specializationName);
	local addSuccess = g_specializationManager:addSpecialization(specializationName, specializationClassName, specializationFilename, nil);
	specializationObject = g_specializationManager:getSpecializationObjectByName(modName .. "."..specializationName);
    for typeName, vehicleType in pairs(g_vehicleTypeManager:getTypes()) do
        local blocked = false;
		if blockSpecializations ~= nil then
			for _, search in pairs(blockSpecializations) do
				local specObject = g_specializationManager:getSpecializationObjectByName(search);
				if SpecializationUtil.hasSpecialization(specObject, vehicleType.specializations) then
                    blocked = true;
                end;
			end;
		end;
		if not blocked then
			if searchedSpecializations ~= nil then
				neededSpec = #searchedSpecializations;
				for _, search in pairs(searchedSpecializations) do
					local specObject = g_specializationManager:getSpecializationObjectByName(search);
					if SpecializationUtil.hasSpecialization(specObject, vehicleType.specializations) then
						hasForNeed = hasForNeed + 1;
					end;
				end;
			end;
			
			if specializationObject.prerequisitesPresent(vehicleType.specializations) then
				if neededSpec <= hasForNeed then
					g_vehicleTypeManager:addSpecialization(typeName, modName .. "." .. specializationName);
					printDebug(string.format("[TiltLosses]: Successfuly registered TiltLosses specialization to vehicle of type %s", typeName), 1, true);
				end;
			end;
		end;
    end;
end;

local registerSpecialization = function()
    local modDesc = loadXMLFile("modDesc", modDir .. "modDesc.xml");
    local key = "modDesc.registerSpecializations.specialization"
    local specializationNumber = 0;
    while true do
        local key = string.format("modDesc.registerSpecializations.registerSpecialization(%d)", specializationNumber);
        if not hasXMLProperty(modDesc, key) then
            break;
        end;
        local name = getXMLString(modDesc, key .. "#name");
        local className = getXMLString(modDesc, key .. "#className");
        local filename = Utils.getFilename(getXMLString(modDesc, key .. "#filename"), modDir);
        local searchedSpecializations = string.split(getXMLString(modDesc, key .. "#searchedSpecializations"), " ");
        local blockSpecializations = string.split(getXMLString(modDesc, key .. "#blockSpecializations"), " ");
        specializationRegister(name, className, filename, searchedSpecializations, blockSpecializations);
        specializationNumber = specializationNumber + 1;
    end;
    delete(modDesc);
end;

TypeManager.validateTypes = Utils.appendedFunction(TypeManager.validateTypes, 
	function(self) 
		if self.typeName == "vehicle" then 
			registerSpecialization();
		end 
	end
);